/*! \file NebulaIRISAPIs.h
This file describes Nebula module IRIS APIs for device.

\copyright Copyright (c) 2023 by Throughtek Co., Ltd. All Rights Reserved.
*/

#ifndef _NEBULA_IRISAPIs_H_
#define _NEBULA_IRISAPIs_H_

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include "NebulaCommon.h"

#define NEBULA_IRIS_API

#define MAX_SESSION_ID_LENGTH 32
#define MAX_URL_LENGTH 256
#define MAX_WIFI_SSID_LENGTH 32
#define MAX_WIFI_PASSWORD_LENGTH 64

typedef struct NebulaIRISBindInfo {
    char session_id[MAX_SESSION_ID_LENGTH + 1];     // reauired, session id for IRIS bind
    char region[MAX_REGION_LENGTH + 1];             // reauired, device region
    char realm[MAX_REALM_LENGTH + 1];               // reauired
    char IRIS_url[MAX_URL_LENGTH + 1];              // optional, Nebula_IRIS_Device_Bind() will use default url if NULL
    char wifi_ssid[MAX_WIFI_SSID_LENGTH + 1];       // optional, wifi ssid for device internet connection
    char wifi_pwd[MAX_WIFI_PASSWORD_LENGTH + 1];    // optional, wifi password for device internet connection
} NebulaIRISBindInfo;

#ifdef TUTK_SUPPORT_IRIS

/**
 * \brief Device parse IRIS bind information from string.
 *
 * \details This function is used by devices to parse IRIS bind information from json format string.
 *
 *
 * \param input_str[in] json string contain IRIS bind information, should be in the format:
 *                      {"r":"IRIS_REALM","s":"IRIS_SESSION_ID","ss":"WIFI_SSID","p":"WIFI_PASSWORD","re":"DEVICE_REGION","h":"IRIS_URL"}
 * \param bind_info[out] output information, see #NebulaIRISBindInfo;
 *
 * \return #NEBULA_ER_NoERROR if parse successfully
 * \return Error code if return value < 0
 *            - #NEBULA_ER_INVALID_ARG The input_str or bind_info is null
 *            - #TUTK_JSON_ER_XXXXX Fail to parse json string
 */
NEBULA_IRIS_API int Nebula_IRIS_Device_Parse_BindInfo(char *input_str, NebulaIRISBindInfo *bind_info);

/**
 * \brief Device post IRIS bind information to server
 *
 * \details This function is used by device to bind client through IRIS.
 *          Device need call Nebula_Device_Login "after" this function.
 *
 * \param bind_info[in] bind information data generate by Nebula_IRIS_Device_Parse_BindInfo()
 * \param public_udid[in] device's public_udid
 * \param credential[in] Nebula bind string generate by Nebula_Device_New_Credential()
 * \param timeout_ms[in] The timeout for this function in unit of millisecond, give 0 means block forever
 * \param abort_flag [in] set *abort_flag to 1 if you need to abort this function
 *
 * \return #NEBULA_ER_NoERROR if post IRIS bind information successfully
 * \return Error code if return value < 0
 *            - #NEBULA_ER_INVALID_ARG The bind_info, public_udid, credential is null or length invalid
 *            - #NEBULA_ER_IRIS_MISSING_PARAM Server error: parameters error
 *            - #NEBULA_ER_IRIS_INVALID_PARAM Server error: parameters error
 *            - #NEBULA_ER_IRIS_VERIFICATION_FAILED Server error: failed to verify
 *            - #NEBULA_ER_IRIS_SESSION_EXPIRED Server error: session is expired
 *            - #NEBULA_ER_TIMEOUT Post data timeout
 *            - #NEBULA_ER_FORCE_STOP The function is aborted by user
 *            - #NEBULA_ER_HTTP_ERROR IRIS Server response error
 *            - #NEBULA_ER_CURL_ERROR An error occurred during the curl connection

 * \attention (1) Recommended value of timeout: 1000 millisecond ~ 30000 millisecond
 *            (2) Once the device bind to a sepecific client successfully, there is no need to do binding again
 *                when device restart.
 */
NEBULA_IRIS_API int Nebula_IRIS_Device_Bind(NebulaIRISBindInfo *bind_info, const char *public_udid, const char *credential, unsigned int timeout_ms, unsigned int *abort_flag);

#endif /* TUTK_SUPPORT_IRIS */

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* _NEBULA_IRISAPIs_H_ */